/*--------------------------------------------------------*/
/* INCLUDE HEADERS
/*--------------------------------------------------------*/
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/*--------------------------------------------------------*/
/* DEFINE MACROS
/*--------------------------------------------------------*/
#define MASTER (processorID == 0)
#define NODE (processorID != 0)
#define LAST_NODE (processorID == numberOfProcessors-1)
#define LEFT_NODE (processorID -1)
#define RIGHT_NODE (processorID + 1)
#define WORLD MPI_COMM_WORLD
#define bool int
#define true 1
#define false 0
/*--------------------------------------------------------*/
/* GLOBALS
/*--------------------------------------------------------*/
bool parallel = true;
int processorID = 0;
int numberOfProcessors = 1;
MPI_Datatype MPI_ProblemInfo;
/*--------------------------------------------------------*/
/* STRUCTURES
/*--------------------------------------------------------*/
typedef struct ProblemInfo {
	int globalNumberOfNodes;
	int numberOfNodesX;
	int numberOfNodesY;
	double endSimulationTime;
	double thermalDiffusivity;
	double CFL;
};
/*--------------------------------------------------------*/
/* FUNCTION PROTOTYPES
/*--------------------------------------------------------*/
bool processUserInput(int &totalNumberOfNodes, double &conductivity, double &endSimulationTime);
bool processUserInput(ProblemInfo &theProblemInfo);
void setupBoundaryConditions(double** theArray, int arraySizeX, int arraySizeY);
void initializeInteriorNodes(double** theArray, int arraySizeX, int arraySizeY);
void initializeArray(double** theArray, int arraySizeX, int arraySizeY);
void MPISetup(int argc, char* argv[]);
int computeClosestDivisibleInteger(int divisibleBy,int inputNumber);
int* makeIntArray(int arraySize);
int** make2DIntArray(int arraySizeX, int arraySizeY);
double* makeDoubleArray(int arraySize);
double** make2DDoubleArray(int arraySizeX, int arraySizeY);
static void buildMPIProblemInfo(ProblemInfo p, MPI_Datatype *MPI_ProblemInfo);
void MPI_BuildDerivedTypes();
